package report

import (
	"strings"
	"testing"

	"github.com/stretchr/testify/assert"
)

func TestBadge(t *testing.T) {
	uu := []struct {
		score int
		e     string
	}{
		{
			90,
			"\x1b[38;5;82;mo          .-'-.     \x1b[0m\n\x1b[38;5;82;m o     __| A    `\\  \x1b[0m\n\x1b[38;5;82;m  o   `-,-`--._   `\\\x1b[0m\n\x1b[38;5;82;m []  .->'  a     `|-'\x1b[0m\n\x1b[38;5;82;m  `=/ (__/_       /  \x1b[0m\n\x1b[38;5;82;m    \\_,    `    _)  \x1b[0m\n\x1b[38;5;82;m       `----;  |     \x1b[0m",
		},
		{
			80,
			"\x1b[38;5;114;mo          .-'-.     \x1b[0m\n\x1b[38;5;114;m o     __| B    `\\  \x1b[0m\n\x1b[38;5;114;m  o   `-,-`--._   `\\\x1b[0m\n\x1b[38;5;114;m []  .->'  a     `|-'\x1b[0m\n\x1b[38;5;114;m  `=/ (__/_       /  \x1b[0m\n\x1b[38;5;114;m    \\_,    `    _)  \x1b[0m\n\x1b[38;5;114;m       `----;  |     \x1b[0m",
		},
		{
			70,
			"\x1b[38;5;122;mo          .-'-.     \x1b[0m\n\x1b[38;5;122;m o     __| C    `\\  \x1b[0m\n\x1b[38;5;122;m  o   `-,-`--._   `\\\x1b[0m\n\x1b[38;5;122;m []  .->'  a     `|-'\x1b[0m\n\x1b[38;5;122;m  `=/ (__/_       /  \x1b[0m\n\x1b[38;5;122;m    \\_,    `    _)  \x1b[0m\n\x1b[38;5;122;m       `----;  |     \x1b[0m",
		},
		{
			60,
			"\x1b[38;5;226;mS          .-'-.     \x1b[0m\n\x1b[38;5;226;m o     __| D    `\\  \x1b[0m\n\x1b[38;5;226;m  S   `-,-`--._   `\\\x1b[0m\n\x1b[38;5;226;m []  .->'  O     `|-'\x1b[0m\n\x1b[38;5;226;m  `=/ (__/_       /  \x1b[0m\n\x1b[38;5;226;m    \\_,    `    _)  \x1b[0m\n\x1b[38;5;226;m       `----;  |     \x1b[0m",
		},
		{
			50,
			"\x1b[38;5;220;mS          .-'-.     \x1b[0m\n\x1b[38;5;220;m o     __| E    `\\  \x1b[0m\n\x1b[38;5;220;m  S   `-,-`--._   `\\\x1b[0m\n\x1b[38;5;220;m []  .->'  O     `|-'\x1b[0m\n\x1b[38;5;220;m  `=/ (__/_       /  \x1b[0m\n\x1b[38;5;220;m    \\_,    `    _)  \x1b[0m\n\x1b[38;5;220;m       `----;  |     \x1b[0m",
		},
		{
			40,
			"\x1b[38;5;196;mS          .-'-.     \x1b[0m\n\x1b[38;5;196;m o     __| F    `\\  \x1b[0m\n\x1b[38;5;196;m  S   `-,-`--._   `\\\x1b[0m\n\x1b[38;5;196;m []  .->'  O     `|-'\x1b[0m\n\x1b[38;5;196;m  `=/ (__/_       /  \x1b[0m\n\x1b[38;5;196;m    \\_,    `    _)  \x1b[0m\n\x1b[38;5;196;m       `----;  |     \x1b[0m",
		},
	}

	s := new(Sanitizer)
	for _, u := range uu {
		assert.Equal(t, u.e, strings.Join(s.Badge(u.score), "\n"))
	}
}
